package cn.com.infosec.jce.provider;

import cn.com.infosec.asn1.pkcs.PKCSObjectIdentifiers;
import cn.com.infosec.asn1.x509.X509ObjectIdentifiers;
import cn.com.infosec.crypto.DSA;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.digests.GOST3411Digest;
import cn.com.infosec.crypto.params.AsymmetricKeyParameter;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import cn.com.infosec.crypto.signers.ECGOST3410Signer;
import cn.com.infosec.crypto.signers.GOST3410Signer;
import cn.com.infosec.jce.interfaces.ECKey;
import cn.com.infosec.jce.interfaces.ECPublicKey;
import cn.com.infosec.jce.interfaces.GOST3410Key;
import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.security.SignatureException;
import java.security.SignatureSpi;
import java.security.spec.AlgorithmParameterSpec;

public class JDKGOST3410Signer extends SignatureSpi implements PKCSObjectIdentifiers, X509ObjectIdentifiers {
   private Digest digest;
   private DSA signer;
   private SecureRandom random;

   protected JDKGOST3410Signer(Digest digest, DSA signer) {
      this.digest = digest;
      this.signer = signer;
   }

   protected void engineInitVerify(PublicKey publicKey) {
      AsymmetricKeyParameter publicKey1;
      if (publicKey instanceof ECPublicKey) {
         publicKey1 = ECUtil.generatePublicKeyParameter(publicKey);
      } else if (publicKey instanceof GOST3410Key) {
         publicKey1 = GOST3410Util.generatePublicKeyParameter(publicKey);
      } else {
         PublicKey var10000;
         boolean var10001;
         try {
            var10000 = JDKKeyFactory.createPublicKeyFromDERStream(publicKey.getEncoded());
         } catch (Exception var5) {
            var10001 = false;
            throw new InvalidKeyException("can't recognise key type in DSA based signer");
         }

         publicKey = var10000;

         boolean var7;
         try {
            var7 = var10000 instanceof ECPublicKey;
         } catch (Exception var4) {
            var10001 = false;
            throw new InvalidKeyException("can't recognise key type in DSA based signer");
         }

         if (!var7) {
            try {
               throw new InvalidKeyException("can't recognise key type in DSA based signer");
            } catch (Exception var2) {
               var10001 = false;
               throw new InvalidKeyException("can't recognise key type in DSA based signer");
            }
         }

         AsymmetricKeyParameter var8;
         try {
            var8 = ECUtil.generatePublicKeyParameter(publicKey);
         } catch (Exception var3) {
            var10001 = false;
            throw new InvalidKeyException("can't recognise key type in DSA based signer");
         }

         publicKey1 = var8;
      }

      this.digest.reset();
      this.signer.init(false, publicKey1);
   }

   protected void engineInitSign(PrivateKey privateKey, SecureRandom random) {
      this.random = random;
      this.engineInitSign(privateKey);
   }

   protected void engineInitSign(PrivateKey privateKey) {
      AsymmetricKeyParameter privateKey1;
      if (privateKey instanceof ECKey) {
         privateKey1 = ECUtil.generatePrivateKeyParameter(privateKey);
      } else {
         privateKey1 = GOST3410Util.generatePrivateKeyParameter(privateKey);
      }

      this.digest.reset();
      SecureRandom var2;
      if ((var2 = this.random) != null) {
         DSA var10000 = this.signer;
         ParametersWithRandom var3;
         var3 = new ParametersWithRandom.<init>(privateKey1, var2);
         var10000.init(true, var3);
      } else {
         this.signer.init(true, privateKey1);
      }

   }

   protected void engineUpdate(byte b) {
      this.digest.update(b);
   }

   protected void engineUpdate(byte[] b, int off, int len) {
      this.digest.update(b, off, len);
   }

   protected byte[] engineSign() {
      JDKGOST3410Signer var10000 = this;
      byte[] var23 = new byte[this.digest.getDigestSize()];
      this.digest.doFinal(var23, 0);

      Exception var28;
      label138: {
         boolean var30;
         byte[] var31;
         try {
            var31 = new byte[64];
         } catch (Exception var22) {
            var28 = var22;
            var30 = false;
            break label138;
         }

         byte[] var1 = var31;

         BigInteger[] var29;
         try {
            var29 = var10000.signer.generateSignature(var23);
            var31 = var29[0].toByteArray();
         } catch (Exception var21) {
            var28 = var21;
            var30 = false;
            break label138;
         }

         var23 = var31;

         byte[] var32;
         try {
            var32 = var29[1].toByteArray();
         } catch (Exception var20) {
            var28 = var20;
            var30 = false;
            break label138;
         }

         byte[] var2 = var32;

         byte var33;
         try {
            var33 = var32[0];
         } catch (Exception var19) {
            var28 = var19;
            var30 = false;
            break label138;
         }

         int var3;
         byte[] var10002;
         int var10003;
         int var10005;
         int var26;
         byte[] var34;
         int var35;
         byte var36;
         if (var33 != 0) {
            var32 = var2;
            var31 = var2;
            var3 = 0;

            try {
               var10003 = var2.length;
            } catch (Exception var18) {
               var28 = var18;
               var30 = false;
               break label138;
            }

            var26 = 32 - var10003;

            try {
               var35 = var31.length;
            } catch (Exception var17) {
               var28 = var17;
               var30 = false;
               break label138;
            }

            int var4 = var35;

            try {
               System.arraycopy(var32, var3, var1, var26, var4);
            } catch (Exception var16) {
               var28 = var16;
               var30 = false;
               break label138;
            }
         } else {
            var32 = var2;
            var36 = 1;
            var10002 = var1;
            var34 = var2;

            try {
               var10005 = var2.length;
            } catch (Exception var15) {
               var28 = var15;
               var30 = false;
               break label138;
            }

            var26 = 32 - (var10005 - 1);

            try {
               var10003 = var34.length;
            } catch (Exception var14) {
               var28 = var14;
               var30 = false;
               break label138;
            }

            --var10003;

            try {
               System.arraycopy(var32, var36, var10002, var26, var10003);
            } catch (Exception var13) {
               var28 = var13;
               var30 = false;
               break label138;
            }
         }

         try {
            var33 = var23[0];
         } catch (Exception var12) {
            var28 = var12;
            var30 = false;
            break label138;
         }

         int var24;
         if (var33 != 0) {
            var32 = var23;
            var31 = var23;
            byte var27 = 0;

            try {
               var10003 = var23.length;
            } catch (Exception var11) {
               var28 = var11;
               var30 = false;
               break label138;
            }

            var24 = 64 - var10003;

            try {
               var35 = var31.length;
            } catch (Exception var10) {
               var28 = var10;
               var30 = false;
               break label138;
            }

            var3 = var35;

            try {
               System.arraycopy(var32, var27, var1, var24, var3);
            } catch (Exception var9) {
               var28 = var9;
               var30 = false;
               break label138;
            }
         } else {
            var32 = var23;
            var36 = 1;
            var10002 = var1;
            var34 = var23;

            try {
               var10005 = var23.length;
            } catch (Exception var8) {
               var28 = var8;
               var30 = false;
               break label138;
            }

            var24 = 64 - (var10005 - 1);

            try {
               var10003 = var34.length;
            } catch (Exception var7) {
               var28 = var7;
               var30 = false;
               break label138;
            }

            --var10003;

            try {
               System.arraycopy(var32, var36, var10002, var24, var10003);
            } catch (Exception var6) {
               var28 = var6;
               var30 = false;
               break label138;
            }
         }

         try {
            return var1;
         } catch (Exception var5) {
            var28 = var5;
            var30 = false;
         }
      }

      Exception var25 = var28;
      throw new SignatureException(var25.toString());
   }

   protected boolean engineVerify(byte[] sigBytes) {
      JDKGOST3410Signer var10000 = this;
      byte[] var10001 = sigBytes;
      byte[] var10002 = sigBytes;
      JDKGOST3410Signer var10003 = this;
      byte[] var9 = new byte[this.digest.getDigestSize()];
      var10003.digest.doFinal(var9, 0);

      boolean var12;
      byte[] var16;
      try {
         var16 = new byte[32];
      } catch (Exception var8) {
         var12 = false;
         throw new SignatureException("error decoding signature bytes.");
      }

      sigBytes = var16;

      try {
         var16 = new byte[32];
      } catch (Exception var7) {
         var12 = false;
         throw new SignatureException("error decoding signature bytes.");
      }

      byte[] var2 = var16;

      BigInteger[] var13;
      try {
         System.arraycopy(var10002, 0, var2, 0, 32);
         System.arraycopy(var10001, 32, sigBytes, 0, 32);
         var13 = new BigInteger[2];
      } catch (Exception var6) {
         var12 = false;
         throw new SignatureException("error decoding signature bytes.");
      }

      BigInteger[] var3;
      BigInteger[] var14 = var3 = var13;
      byte var17 = 0;

      try {
         var14[var17] = new BigInteger(1, sigBytes);
      } catch (Exception var5) {
         var12 = false;
         throw new SignatureException("error decoding signature bytes.");
      }

      byte var15 = 1;

      try {
         var13[var15] = new BigInteger(1, var2);
      } catch (Exception var4) {
         var12 = false;
         throw new SignatureException("error decoding signature bytes.");
      }

      DSA var11 = var10000.signer;
      var10001 = var9;
      BigInteger var10 = var3[0];
      return var11.verifySignature(var10001, var10, var3[1]);
   }

   protected void engineSetParameter(AlgorithmParameterSpec params) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   /** @deprecated */
   protected void engineSetParameter(String param, Object value) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   /** @deprecated */
   protected Object engineGetParameter(String param) {
      throw new UnsupportedOperationException("engineSetParameter unsupported");
   }

   public static class ecgost3410 extends JDKGOST3410Signer {
      public ecgost3410() {
         ecgost3410 var10000 = this;
         GOST3411Digest var2;
         var2 = new GOST3411Digest.<init>();
         ECGOST3410Signer var1;
         var1 = new ECGOST3410Signer.<init>();
         var10000.<init>(var2, var1);
      }
   }

   public static class gost3410 extends JDKGOST3410Signer {
      public gost3410() {
         gost3410 var10000 = this;
         GOST3411Digest var2;
         var2 = new GOST3411Digest.<init>();
         GOST3410Signer var1;
         var1 = new GOST3410Signer.<init>();
         var10000.<init>(var2, var1);
      }
   }
}
